{*************************************************************************}
{*   Win32SysInfo                                                        *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Engine\Kernel\Platforms\Win32\Win32SysInfo                          *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 12th August 2003  (12.08.2003)                             }
{*************************************************************************}

{Contains function for retrieve CPU, memory and other system info under
 Win32.}

unit Win32SysInfo;

{$I Compiler.inc}

interface

uses
  Kernel, Windows, SysUtils, Platforms 
{$IFNDEF MODLIBRARY}
  , X86CpuInfo, Win32Registry
{$ENDIF};


procedure GetCPUInfo(var Info: TCPUInfo);
procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
procedure GetMemoryInfo(var Info: TMemoryInfo);


{*************************************************************************}
{                             Implementation                              }
{*************************************************************************}


implementation

const

  VER_NT_WORKSTATION = $0000001;
  VER_NT_DOMAIN_CONTROLLER = $0000002;
  VER_NT_SERVER = $0000003;

  VER_SUITE_SMALLBUSINESS = $00000001;
  VER_SUITE_ENTERPRISE = $00000002;
  VER_SUITE_BACKOFFICE = $00000004;
  VER_SUITE_COMMUNICATIONS = $00000008;
  VER_SUITE_TERMINAL = $00000010;
  VER_SUITE_SMALLBUSINESS_RESTRICTED = $00000020;
  VER_SUITE_EMBEDDEDNT = $00000040;
  VER_SUITE_DATACENTER = $00000080;
  VER_SUITE_SINGLEUSERTS = $00000100;
  VER_SUITE_PERSONAL = $00000200;
  VER_SUITE_SERVERAPPLIANCE = $00000400;

type

  POSVersionInfoEx = ^TOSVersionInfoEx;
  TOSVersionInfoEx = record
    dwOSVersionInfoSize: DWORD;
    dwMajorVersion: DWORD;
    dwMinorVersion: DWORD;
    dwBuildNumber: DWORD;
    dwPlatformId: DWORD;
    szCSDVersion: array[0..127] of Char;
    wServicePackMajor: Word;
    wServicePackMinor: Word;
    wSuiteMask: Word;
    wProductType: Byte;
    wReserved: Byte;
  end;

function GetVersionEx(VI: POSVersionInfoEx): BOOL; stdcall;
  external kernel32 name 'GetVersionExA';

procedure GetCPUInfo(var Info: TCpuInfo);
var
{$IFNDEF MODLIBRARY}
  Key: NString;
  X86Info: TCPUInfo;
{$ENDIF}
  SystemInfo: TSystemInfo;

{$IFNDEF MODLIBRARY}
  function GetCPUFreq: Long;
  var
    T0, T1: TInt64Rec;
    CountFreq: TInt64Rec;
    Freq, Freq2, Freq3, Total: Long;
    TotalCycles, Cycles: Int64;
    Stamp0, Stamp1: Int64;
    TotalTicks, Ticks, NormFreq, RawFreq: ULong;
    Tries, Priority: Long;
    Thread: THandle;

    function ReadTimeStampCounter: Int64;
    asm
        DW      $310F
    end;

    function RoundFrequency(const Frequency: Integer): Integer;
    const
      NF: array[0..8] of Integer = (0, 20, 33, 50, 60, 66, 80, 90, 100);
    var
      Freq, RF: Integer;
      I: Byte;
      Hi, Lo: Byte;
    begin
      RF := 0;
      Freq := Frequency mod 100;
      for I := 0 to 8 do
      begin
        if Freq < NF[I] then
        begin
          Hi := I;
          Lo := I - 1;
          if (NF[Hi] - Freq) > (Freq - NF[Lo]) then
            RF := NF[Lo] - Freq
          else
            RF := NF[Hi] - Freq;
          Break;
        end;
      end;
      Result := Frequency + RF;
    end;

  begin
    Stamp0 := 0;
    Stamp1 := 0;
    Freq := 0;
    Freq2 := 0;
    Freq3 := 0;
    Tries := 0;
    TotalCycles := 0;
    TotalTicks := 0;
    Total := 0;
    Thread := GetCurrentThread();
    QueryPerformanceFrequency(CountFreq.Int64Value);
    while ((Tries < 3) or ((Tries < 20) and ((Abs(3 * Freq - Total) > 3) or
      (Abs(3 * Freq2 - Total) > 3) or (Abs(3 * Freq3 - Total) > 3)))) do
    begin
      Inc(Tries);
      Freq3 := Freq2;
      Freq2 := Freq;
      QueryPerformanceCounter(T0.Int64Value);
      T1 := T0;
      Priority := GetThreadPriority(Thread);
      if Priority <> THREAD_PRIORITY_ERROR_RETURN then
        SetThreadPriority(Thread, THREAD_PRIORITY_TIME_CRITICAL);
      try
        while (T1.ULongs[0] - T0.ULongs[0]) < 50 do
        begin
          QueryPerformanceCounter(T1.Int64Value);
          Stamp0 := ReadTimeStampCounter;
        end;
        T0 := T1;
        while (T1.ULongs[0] - T0.ULongs[0]) < 1000 do
        begin
          QueryPerformanceCounter(T1.Int64Value);
          Stamp1 := ReadTimeStampCounter;
        end;
      finally
        if Priority <> THREAD_PRIORITY_ERROR_RETURN then
          SetThreadPriority(Thread, Priority);
      end;
      Cycles := Stamp1 - Stamp0;
      Ticks := T1.ULongs[0] - T0.ULongs[0];
      Ticks := Ticks * 100000;
      Ticks := Ticks div (CountFreq.ULongs[0] div 10);
      TotalTicks := TotalTicks + Ticks;
      TotalCycles := TotalCycles + Cycles;
      Freq := Cycles div Ticks;
      Total := Freq + Freq2 + Freq3;
    end;
    Freq3 := (TotalCycles * 10) div TotalTicks;
    Freq2 := (TotalCycles * 100) div TotalTicks;
    if Freq2 - (Freq3 * 10) >= 6 then
      Inc(Freq3);
    RawFreq := TotalCycles div TotalTicks;
    NormFreq := RawFreq;
    Freq := RawFreq * 10;
    if (Freq3 - Freq) >= 6 then
      Inc(NormFreq);
    Result := RoundFrequency(NormFreq);
  end;
{$ENDIF}

begin
  Finalize(Info);
  ZeroMemory(@Info, SizeOf(Info));

{$IFNDEF MODLIBRARY}
  // WindowsNT only
  Key := 'Hardware\Description\System\CentralProcessor\0';
  if RegKeyExists(HKEY_LOCAL_MACHINE, Key) then
  try
    Info.Frequency := RegReadInt(HKEY_LOCAL_MACHINE, Key, '~MHz');
    // AMD and P4
    Info.Name := Trim(RegReadStr(HKEY_LOCAL_MACHINE, Key, 'ProcessorNameString'));
  except
    // show no exceptions !!
  end;

  if Info.Frequency = 0 then Info.Frequency := Round(GetCPUFreq);
  X86CpuInfo.GetX86CPUInfo(X86Info);
  if Info.Name = '' then Info.Name := X86Info.Name;
  Info.Vendor := X86Info.Vendor;
  Info.Features := X86Info.Features;
  Info.Family := X86Info.Family;
  Info.Model := X86Info.Model;
  Info.Stepping := X86Info.Stepping;
  Info.Cache := X86Info.Cache;
{$ENDIF}

  ZeroMemory(@SystemInfo, SizeOf(SystemInfo));
  GetSystemInfo(SystemInfo);
  Info.ProcessorCount := SystemInfo.dwNumberOfProcessors;
end;

procedure GetOSInfo(var Version: TVersionInfo; var Name: NString);
var
  OS: TOSVersionInfoEx;
begin
  ZeroMemory(@OS, SizeOf(OS));
  OS.dwOSVersionInfoSize := SizeOf(OS);
  GetVersionEx(@OS);
  Version.Major := OS.dwMajorVersion;
  Version.Minor := OS.dwMinorVersion;
  Version.Build := OS.dwBuildNumber;

  Name := 'MS Windows';
  case OS.dwPlatformId of
    VER_PLATFORM_WIN32_WINDOWS:
      case Version.Minor of
        0..9:
          if Trim(OS.szCSDVersion) = 'B' then
            Name := 'MS Windows 95 OSR2'
          else
            Name := 'MS Windows 95';
        10..89:
          if Version.Build <> 2222 then
            Name := 'MS Windows 98'
          else
            Name := 'MS Windows 98 SE';
        90: Name := 'MS Windows Me';
      end;
    VER_PLATFORM_WIN32_NT:
      case Version.Major of
        3: case Version.Minor of
            1: Name := 'MS Windows NT 3.1';
            5: Name := 'MS Windows NT 3.5';
            51: Name := 'MS Windows NT 3.51';
          end;
        4: Name := 'MS Windows NT 4.0';
        5: case Version.Minor of
            0:
              begin
                Name := 'MS Windows 2000';
                if (OS.wProductType = VER_NT_SERVER) then
                begin
                  if (OS.wSuiteMask and VER_SUITE_DATACENTER) =
                    VER_SUITE_DATACENTER then
                    Name := Name + ' Data Center'
                  else
                    if (OS.wSuiteMask and VER_SUITE_ENTERPRISE) =
                      VER_SUITE_ENTERPRISE then
                      Name := Name + ' Advanced Server'
                    else
                      Name := Name + ' Server';
                end
                else
                  if (OS.wProductType = VER_NT_WORKSTATION) then
                    Name := Name + ' Professional';
              end;
            1:
              begin
                Name := 'MS Windows XP';
                if (OS.wProductType = VER_NT_WORKSTATION) then
                begin
                  if (OS.wSuiteMask and VER_SUITE_PERSONAL) = VER_SUITE_PERSONAL
                    then
                    Name := Name + ' Personal'
                  else
                    Name := Name + ' Professional';
                end;
              end;
            2: Name := 'MS Windows Server 2003';
          end;
      end;
  end;
end;

procedure GetMemoryInfo(var Info: TMemoryInfo);
var
  MS: TMemoryStatus;
begin
  MS.dwLength := SizeOf(MS);
  GlobalMemoryStatus(MS);
  Info.CurrentLoad := MS.dwMemoryLoad;
  Info.PhysicalTotal := MS.dwTotalPhys div 1024 + 640;
  Info.PhysicalFree := MS.dwAvailPhys div 1024;
  Info.PageFileTotal := MS.dwTotalPageFile div 1024;
  Info.PageFileFree := MS.dwAvailPageFile div 1024;
  Info.VirtualTotal := MS.dwTotalVirtual div 1024;
  Info.VirtualFree := MS.dwAvailVirtual div 1024;
end;


end.

